@@ -1,16 +1,44 @@
 #!/bin/bash
 # Copyright 1999-2003 Gentoo Technologies, Inc.
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /home/cvsroot/gentoo-src/portage/bin/ebuild.sh,v 1.118 2003/03/11 11:33:07 carpaski Exp $
+# $Header: /home/cvsroot/gentoo-src/portage/bin/ebuild.sh,v 1.135 2003/07/22 07:19:04 carpaski Exp $
 
 if [ "$*" != "depend" ] && [ "$*" != "clean" ]; then
+	if [ -f ${T}/successful ]; then
+		rm -f ${T}/successful
+	fi
+
+	# Hurray for per-ebuild logging.
+	if [ ! -z "${PORT_LOGDIR}" ]; then
+		if [ -z "${PORT_LOGGING}" ]; then
+			export PORT_LOGGING=1
+			export SANDBOX_WRITE="$SANDBOX_WRITE:${PORT_LOGDIR}"
+			install -d ${PORT_LOGDIR} &>/dev/null
+			chown root:portage ${PORT_LOGDIR} &>/dev/null
+			chmod g+rwxs ${PORT_LOGDIR} &> /dev/null
+			touch "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log" &> /dev/null
+			chmod g+w "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log" &> /dev/null
+			echo "$*" >> "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log"
+			$0 $* 2>&1 | tee -a "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log"
+			if [ "$?" != "0" ]; then
+				rm -f ${T}/successful
+				exit 1
+			fi
+			if [ -f ${T}/successful ]; then
+				rm -f ${T}/successful
+				exit 0
+			else
+				exit 1
+			fi
+		fi
+	fi
+
 	if [ -f "${T}/environment" ]; then
 		source "${T}/environment" &>/dev/null
 	fi
 fi
 
-if [ -n "$#" ]
-then
+if [ -n "$#" ]; then
 	ARGS="${*}"
 fi
 
@@ -26,7 +54,9 @@ alias assert='_retval=$?; [ $_retval = 0 ] || diefunc "$FUNCNAME" "$LINENO" "$_r
 
 OCC="$CC"
 OCXX="$CXX"
-source /etc/profile.env > /dev/null 2>&1
+if [ "$USERLAND" == "GNU" ]; then
+	source /etc/profile.env &>/dev/null
+fi
 [ ! -z "$OCC" ] && export CC="$OCC"
 [ ! -z "$OCXX" ] && export CXX="$OCXX"
 
@@ -37,10 +67,14 @@ export PATH="/sbin:/usr/sbin:/usr/lib/portage/bin:/bin:/usr/bin:${ROOTPATH}"
 source /usr/lib/portage/bin/extra_functions.sh
 
 if [ -e /etc/init.d/functions.sh ]; then
-	source /etc/init.d/functions.sh > /dev/null 2>&1
+	source /etc/init.d/functions.sh  &>/dev/null
 elif [ -e /etc/rc.d/config/functions ];	then
-	source /etc/rc.d/config/functions > /dev/null 2>&1
+	source /etc/rc.d/config/functions &>/dev/null
+else
+	#Mac OS X
+	source /usr/lib/portage/bin/functions.sh &>/dev/null
 fi
+
 esyslog() {
 	# Custom version of esyslog() to take care of the "Red Star" bug.
 	# MUST follow functions.sh to override the "" parameter problem.
@@ -49,12 +83,9 @@ esyslog() {
 
 use() {
 	local x
-	for x in ${USE}
-	do
-		if [ "${x}" = "${1}" ]
-		then
-			#tty --quiet < /dev/stdout || echo "${x}"
-			echo "${x}"
+	for x in ${USE}; do
+		if [ "${x}" == "${1}" ]; then
+			tty --quiet < /dev/stdout || echo "${x}"
 			return 0
 		fi
 	done
@@ -68,12 +99,9 @@ has() {
 	me=$1
 	shift
 	
-	for x in $@
-	do
-		if [ "${x}" = "${me}" ]
-		then
-			#tty --quiet < /dev/stdout || echo "${x}"
-			echo "${x}"
+	for x in "$@"; do
+		if [ "${x}" == "${me}" ]; then
+			tty --quiet < /dev/stdout || echo "${x}"
 			return 0
 		fi
 	done
@@ -83,7 +111,7 @@ has() {
 has_version() {
 	# return shell-true/shell-false if exists.
 	# Takes single depend-type atoms.
-	if /usr/lib/portage/bin/portageq 'has_version' ${ROOT} $1; then
+	if /usr/lib/portage/bin/portageq 'has_version' "${ROOT}" "$1"; then
 		return 0
 	else
 		return 1
@@ -93,12 +121,19 @@ has_version() {
 best_version() {
 	# returns the best/most-current match.
 	# Takes single depend-type atoms.
-	/usr/lib/portage/bin/portageq 'best_version' ${ROOT} $1
+	/usr/lib/portage/bin/portageq 'best_version' "${ROOT}" "$1"
 }
 
 use_with() {
 	if [ -z "$1" ]; then
-		die "use_with() called without parameter."
+		echo "!!! use_with() called without a parameter." >&2
+		echo "!!! use_with <USEFLAG> [<flagname> [value]]" >&2
+		return
+	fi
+
+	local UW_SUFFIX=""	
+	if [ ! -z "${3}" ]; then
+		UW_SUFFIX="=${3}"
 	fi
 
 	local UWORD="$2"
@@ -107,7 +142,7 @@ use_with() {
 	fi
 	
 	if use $1 &>/dev/null; then
-		echo "--with-${UWORD}"
+		echo "--with-${UWORD}${UW_SUFFIX}"
 		return 0
 	else
 		echo "--without-${UWORD}"
@@ -117,7 +152,14 @@ use_with() {
 
 use_enable() {
 	if [ -z "$1" ]; then
-		die "use_with() called without parameter."
+		echo "!!! use_enable() called without a parameter." >&2
+		echo "!!! use_enable <USEFLAG> [<flagname> [value]]" >&2
+		return
+	fi
+
+	local UE_SUFFIX=""	
+	if [ ! -z "${3}" ]; then
+		UE_SUFFIX="=${3}"
 	fi
 
 	local UWORD="$2"
@@ -126,7 +168,7 @@ use_enable() {
 	fi
 	
 	if use $1 &>/dev/null; then
-		echo "--enable-${UWORD}"
+		echo "--enable-${UWORD}${UE_SUFFIX}"
 		return 0
 	else
 		echo "--disable-${UWORD}"
@@ -162,8 +204,7 @@ export MOPREFIX=${PN}
 
 check_KV()
 {
-	if [ x"${KV}" = x ]
-	then
+	if [ -z "${KV}" ]; then
 		eerror ""
 		eerror "Could not determine your kernel version."
 		eerror "Make sure that you have /usr/src/linux symlink."
@@ -179,11 +220,10 @@ check_KV()
 # adds ".keep" files so that dirs aren't auto-cleaned
 keepdir()
 {
-	dodir $*
+	dodir "$@"
 	local x
-	for x in $*
-	do
-		[ ! -e ${D}/${x}/.keep -o -w ${D}/${x}/.keep ] && touch ${D}/${x}/.keep
+	for x in "$@"; do
+		touch ${D}/${x}/.keep || die "Failed to create .keep in ${D}/${x}"
 	done
 }
 
@@ -216,48 +256,55 @@ unpack() {
 	local x
 	local y
 	local myfail
-	
-	for x in $@
-	do
+	local tarvars
+
+	if [ "$USERLAND" == "BSD" ]; then
+		tarvars=""
+	else
+		tarvars="--no-same-owner"	
+	fi	
+
+	for x in "$@"; do
 		myfail="failure unpacking ${x}"
 		echo ">>> Unpacking ${x} to $(pwd)"
 		y="$(echo $x | sed 's:.*\.\(tar\)\.[a-zA-Z0-9]*:\1:')"
+
 		case "${x##*.}" in
-		tar) 
-			tar x --no-same-owner -f ${DISTDIR}/${x} || die "$myfail"
-			;;
-		tgz) 
-			tar xz --no-same-owner -f ${DISTDIR}/${x} || die "$myfail"
-			;;
-		tbz2) 
-			tar xj --no-same-owner -f ${DISTDIR}/${x} || die "$myfail"
-			;;
-		ZIP|zip) 
-			unzip -qo ${DISTDIR}/${x} || die "$myfail"
-			;;
-		gz|Z|z) 
-			if [ "${y}" == "tar" ]; then
-				tar xz --no-same-owner -f ${DISTDIR}/${x} || die "$myfail"
-			else
-				gzip -dc ${DISTDIR}/${x} > ${x%.*} || die "$myfail"
-			fi
-			;;
-		bz2) 
-			if [ "${y}" == "tar" ]; then
-				tar xj --no-same-owner -f ${DISTDIR}/${x} || die "$myfail"
-			else
-				bzip2 -dc ${DISTDIR}/${x} > ${x%.*} || die "$myfail"
-			fi
-			;;
-		*)
-			echo "unpack ${x}: file format not recognized. Ignoring."
-			;;
+			tar) 
+				tar ${tarvars} -xf ${DISTDIR}/${x} || die "$myfail"
+				;;
+			tgz) 
+				tar ${tarvars} -xzf ${DISTDIR}/${x} || die "$myfail"
+				;;
+			tbz2) 
+				bzip2 -dc ${DISTDIR}/${x} | tar ${tarvars} -xf - || die "$myfail"
+				;;
+			ZIP|zip) 
+				unzip -qo ${DISTDIR}/${x} || die "$myfail"
+				;;
+			gz|Z|z) 
+				if [ "${y}" == "tar" ]; then
+					tar ${tarvars} -xzf ${DISTDIR}/${x} || die "$myfail"
+				else
+					gzip -dc ${DISTDIR}/${x} > ${x%.*} || die "$myfail"
+				fi
+				;;
+			bz2) 
+				if [ "${y}" == "tar" ]; then
+					bzip2 -dc ${DISTDIR}/${x} | tar ${tarvars} -xf - || die "$myfail"
+				else
+					bzip2 -dc ${DISTDIR}/${x} > ${x%.*} || die "$myfail"
+				fi
+				;;
+			*)
+				echo "unpack ${x}: file format not recognized. Ignoring."
+				;;
 		esac
 	done
 }
 
 econf() {
-	if [ -x ./configure ] ; then
+	if [ -x ./configure ]; then
 		if [ ! -z "${CBUILD}" ]; then
 			EXTRA_ECONF="--build=${CBUILD} ${EXTRA_ECONF}"
 		fi
@@ -278,6 +325,15 @@ econf() {
 
 einstall() {
 	if [ -f ./[mM]akefile -o -f ./GNUmakefile ] ; then
+		if [ ! -z "${PORTAGE_DEBUG}" ]; then
+			make -n prefix=${D}/usr \
+			    datadir=${D}/usr/share \
+			    infodir=${D}/usr/share/info \
+		  	  localstatedir=${D}/var/lib \
+			    mandir=${D}/usr/share/man \
+			    sysconfdir=${D}/etc \
+			    "$@" install
+		fi
 		make prefix=${D}/usr \
 		    datadir=${D}/usr/share \
 		    infodir=${D}/usr/share/info \
@@ -303,22 +359,19 @@ pkg_nofetch()
 	for MYFILE in `echo ${SRC_URI}`; do
 		echo "!!!   $MYFILE"
 	done
-	return 
 }
 
 src_unpack() { 
-	if [ "${A}" != "" ]
-	then
+	if [ "${A}" != "" ]; then
 		unpack ${A}
 	fi	
 }
 
 src_compile() { 
-	if [ -x ./configure ] ; then
+	if [ -x ./configure ]; then
 		econf 
 		emake || die "emake failed"
 	fi
-	return 
 }
 
 src_install() 
@@ -372,73 +425,65 @@ END
 
 dyn_setup()
 {
-	# The next bit is to ease the broken pkg_postrm()'s
-	# some of the gcc ebuilds have that nuke the new
-	# /lib/cpp and /usr/bin/cc wrappers ...
+	if [ "$USERLAND" == "Linux" ]; then	
+		# The next bit is to ease the broken pkg_postrm()'s
+		# some of the gcc ebuilds have that nuke the new
+		# /lib/cpp and /usr/bin/cc wrappers ...
 	
-	# Make sure we can have it disabled somehow ....
-	if [ "${DISABLE_GEN_GCC_WRAPPERS}" != "yes" ]
-	then
-		# Create /lib/cpp if missing or a symlink
-		if [ -L /lib/cpp -o ! -e /lib/cpp ]
-		then
-			[ -L /lib/cpp ] && rm -f /lib/cpp
-			gen_wrapper /lib/cpp cpp
-		fi
-		# Create /usr/bin/cc if missing for a symlink
-		if [ -L /usr/bin/cc -o ! -e /usr/bin/cc ]
-		then
-			[ -L /usr/bin/cc ] && rm -f /usr/bin/cc
-			gen_wrapper /usr/bin/cc gcc
+		# Make sure we can have it disabled somehow ....
+		if [ "${DISABLE_GEN_GCC_WRAPPERS}" != "yes" ]; then
+			# Create /lib/cpp if missing or a symlink
+			if [ -L /lib/cpp -o ! -e /lib/cpp ]; then
+				[ -L /lib/cpp ] && rm -f /lib/cpp
+				gen_wrapper /lib/cpp cpp
+			fi
+			# Create /usr/bin/cc if missing for a symlink
+			if [ -L /usr/bin/cc -o ! -e /usr/bin/cc ]; then
+				[ -L /usr/bin/cc ] && rm -f /usr/bin/cc
+				gen_wrapper /usr/bin/cc gcc
+			fi
 		fi
 	fi
-
 	pkg_setup || die "pkg_setup function failed; exiting."
 }
 
 dyn_unpack() {
 	trap "abort_unpack" SIGINT SIGQUIT
 	local newstuff="no"
-	if [ -e ${WORKDIR} ]
-	then
+	if [ -e "${WORKDIR}" ]; then
 		local x
 		local checkme
-		for x in ${AA}
-		do
+		for x in ${AA}; do
 			echo ">>> Checking ${x}'s mtime..."
-			if [ ${DISTDIR}/${x} -nt ${WORKDIR} ]
-			then
+			if [ "${DISTDIR}/${x}" -nt "${WORKDIR}" ]; then
 				echo ">>> ${x} has been updated; recreating WORKDIR..."
 				newstuff="yes"
-				rm -rf ${WORKDIR}
+				rm -rf "${WORKDIR}"
 				break
 			fi
 		done
-		if [ ${EBUILD} -nt ${WORKDIR} ]
-		then
+		if [ ${EBUILD} -nt "${WORKDIR}" ]; then
 			echo ">>> ${EBUILD} has been updated; recreating WORKDIR..."
 			newstuff="yes"
-			rm -rf ${WORKDIR}
-		elif [ ! -f ${BUILDDIR}/.unpacked ]; then
+			rm -rf "${WORKDIR}"
+		elif [ ! -f "${BUILDDIR}/.unpacked" ]; then
 			echo ">>> Not marked as unpacked; recreating WORKDIR..."
 			newstuff="yes"
-			rm -rf ${WORKDIR}
+			rm -rf "${WORKDIR}"
 		fi
 	fi
-	if [ -e ${WORKDIR} ]
-	then
-		if [ "$newstuff" = "no" ]
-		then
+	if [ -e "${WORKDIR}" ]; then
+		if [ "$newstuff" == "no" ]; then
 			echo ">>> WORKDIR is up-to-date, keeping..."
 			return 0
 		fi
 	fi
-	install -m0700 -d ${WORKDIR}
-	[ -d "$WORKDIR" ] && cd ${WORKDIR}
+	
+	install -m0700 -d "${WORKDIR}" || die "Failed to create dir '${WORKDIR}'"
+	[ -d "$WORKDIR" ] && cd "${WORKDIR}"
 	echo ">>> Unpacking source..."
 	src_unpack
-	touch ${BUILDDIR}/.unpacked
-	#|| abort_unpack "fail"
+	touch ${BUILDDIR}/.unpacked || die "IO Failure -- Failed 'touch .unpacked' in BUILDIR"
 	echo ">>> Source unpacked."
 	cd $BUILDDIR
 	trap SIGINT SIGQUIT
@@ -450,91 +495,82 @@ dyn_clean() {
 
 	if ! has keeptemp $FEATURES; then
 		rm -rf ${T}/*
+	else
+		mv ${T}/environment ${T}/environment.keeptemp
 	fi
 
 	if ! has keepwork $FEATURES; then
 		rm -rf ${BUILDDIR}/.compiled
 		rm -rf ${BUILDDIR}/.unpacked
-		rm -rf ${WORKDIR} 
+		rm -rf "${WORKDIR}"
 	fi
 
 	if [ -f ${BUILDDIR}/.unpacked ]; then
-		find ${BUILDDIR} -type d ! -regex "^${WORKDIR}" | sort -r | xargs rmdir &>/dev/null
+		find ${BUILDDIR} -type d ! -regex "^${WORKDIR}" | sort -r | tr "\n" "\0" | $XARGS -0 rmdir &>/dev/null
 	fi
 	true
 }
 
 into() {
-	if [ $1 = "/" ]
-	then
+	if [ $1 == "/" ]; then
 		export DESTTREE=""
 	else
 		export DESTTREE=$1
-		if [ ! -d ${D}${DESTTREE} ]
-		then
-			install -d ${D}${DESTTREE}
+		if [ ! -d "${D}${DESTTREE}" ]; then
+			install -d "${D}${DESTTREE}"
 		fi
 	fi
 }
 
 insinto() {
-	if [ $1 = "/" ]
-	then
+	if [ "$1" == "/" ]; then
 		export INSDESTTREE=""
 	else
 		export INSDESTTREE=$1
-		if [ ! -d ${D}${INSDESTTREE} ]
-		then
-			install -d ${D}${INSDESTTREE}
+		if [ ! -d "${D}${INSDESTTREE}" ]; then
+			install -d "${D}${INSDESTTREE}"
 		fi
 	fi
 }
 
 exeinto() {
-	if [ $1 = "/" ]
-	then
+	if [ "$1" == "/" ]; then
 		export EXEDESTTREE=""
 	else
-		export EXEDESTTREE=$1
-		if [ ! -d ${D}${EXEDESTTREE} ]
-		then
-			install -d ${D}${EXEDESTTREE}
+		export EXEDESTTREE="$1"
+		if [ ! -d "${D}${EXEDESTTREE}" ]; then
+			install -d "${D}${EXEDESTTREE}"
 		fi
 	fi
 }
 
 docinto() {
-	if [ $1 = "/" ]
-	then
+	if [ "$1" == "/" ]; then
 		export DOCDESTTREE=""
 	else
-		export DOCDESTTREE=$1
-		if [ ! -d ${D}usr/share/doc/${PF}/${DOCDESTTREE} ]
-		then
-			install -d ${D}usr/share/doc/${PF}/${DOCDESTTREE} 
+		export DOCDESTTREE="$1"
+		if [ ! -d "${D}usr/share/doc/${PF}/${DOCDESTTREE}" ]; then
+			install -d "${D}usr/share/doc/${PF}/${DOCDESTTREE}"
 		fi
 	fi
 }
 
 insopts() {
 	INSOPTIONS=""
-	for x in $*
-	do
+	for x in $*; do
 		#if we have a debug build, let's not strip anything
-		if [ -n "$DEBUGBUILD" ] && [ "$x" = "-s" ]
-		then
+		if has nostrip $FEATURES $RESTRICT && [ "$x" == "-s" ]; then
 			continue
  		else
 			INSOPTIONS="$INSOPTIONS $x"
-	fi
+		fi
 	done
 	export INSOPTIONS
 }
 
 diropts() {
 	DIROPTIONS=""
-	for x in $*
-	do
+	for x in $*; do
 		DIROPTIONS="${DIROPTIONS} $x"
 	done
 	export DIROPTIONS
@@ -542,11 +578,9 @@ diropts() {
 
 exeopts() {
 	EXEOPTIONS=""
-	for x in $*
-	do
+	for x in $*; do
 		#if we have a debug build, let's not strip anything
-		if [ -n "$DEBUGBUILD" ] && [ "$x" = "-s" ]
-		then
+		if has nostrip $FEATURES $RESTRICT && [ "$x" == "-s" ]; then
 			continue
 		else
 			EXEOPTIONS="$EXEOPTIONS $x"
@@ -557,11 +591,9 @@ exeopts() {
 
 libopts() {
 	LIBOPTIONS=""
-	for x in $*
-	do
+	for x in $*; do
 		#if we have a debug build, let's not strip anything
-		if [ -n "$DEBUGBUILD" ] && [ "$x" = "-s" ]
-		then
+		if has nostrip $FEATURES $RESTRICT && [ "$x" == "-s" ]; then
 			continue
 		else
 			LIBOPTIONS="$LIBOPTIONS $x"
@@ -572,8 +604,7 @@ libopts() {
 
 abort_handler() {
 	local msg
-	if [ "$2" != "fail" ]
-	then
+	if [ "$2" != "fail" ]; then
 		msg="${EBUILD}: ${1} aborted; exiting."
 	else
 		msg="${EBUILD}: ${1} failed; exiting."
@@ -588,7 +619,7 @@ abort_handler() {
 
 abort_compile() {
 	abort_handler "src_compile" $1
-	rm -f ${BUILDDIR}/compiled
+	rm -f ${BUILDDIR}/.compiled
 	exit 1
 }
 
@@ -614,30 +645,46 @@ abort_install() {
 
 dyn_compile() {
 	trap "abort_compile" SIGINT SIGQUIT
-	export CFLAGS CXXFLAGS LIBCFLAGS LIBCXXFLAGS
+	[ "${CFLAGS-unset}"      != "unset" ] && export CFLAGS
+	[ "${CXXFLAGS-unset}"    != "unset" ] && export CXXFLAGS
+	[ "${LIBCFLAGS-unset}"   != "unset" ] && export LIBCFLAGS
+	[ "${LIBCXXFLAGS-unset}" != "unset" ] && export LIBCXXFLAGS
+	[ "${LDFLAGS-unset}"     != "unset" ] && export LDFLAGS
+	[ "${ASFLAGS-unset}"     != "unset" ] && export ASFLAGS
+
 	if has noauto $FEATURES &>/dev/null && [ ! -f ${BUILDDIR}/.unpacked ]; then
 		echo
 		echo "!!! We apparently haven't unpacked... This is probably not what you"
 		echo "!!! want to be doing... You are using FEATURES=noauto so I'll assume"
 		echo "!!! that you know what you are doing... You have 5 seconds to abort..."
 		echo
-		echo -ne "\a"; sleep 0.25 ;	echo -ne "\a"; sleep 0.25
-		echo -ne "\a"; sleep 0.25 ;	echo -ne "\a"; sleep 0.25
-		echo -ne "\a"; sleep 0.25 ;	echo -ne "\a"; sleep 0.25
-		echo -ne "\a"; sleep 0.25 ;	echo -ne "\a"; sleep 0.25
+
+		echo -ne "\a"; sleep 0.25 &>/dev/null; echo -ne "\a"; sleep 0.25 &>/dev/null
+		echo -ne "\a"; sleep 0.25 &>/dev/null; echo -ne "\a"; sleep 0.25 &>/dev/null
+		echo -ne "\a"; sleep 0.25 &>/dev/null; echo -ne "\a"; sleep 0.25 &>/dev/null
+		echo -ne "\a"; sleep 0.25 &>/dev/null; echo -ne "\a"; sleep 0.25 &>/dev/null
+
+		echo -ne "\a"; sleep 0,25 &>/dev/null; echo -ne "\a"; sleep 0,25 &>/dev/null
+		echo -ne "\a"; sleep 0,25 &>/dev/null; echo -ne "\a"; sleep 0,25 &>/dev/null
+		echo -ne "\a"; sleep 0,25 &>/dev/null; echo -ne "\a"; sleep 0,25 &>/dev/null
+		echo -ne "\a"; sleep 0,25 &>/dev/null; echo -ne "\a"; sleep 0,25 &>/dev/null
 		sleep 3
 	fi
+
+	cd ${BUILDDIR}
+	if [ ! -e "build-info" ];	then
+		mkdir build-info
+	fi
+	cp ${EBUILD} build-info/${PF}.ebuild
 	
-	if [ ${BUILDDIR}/.compiled -nt ${WORKDIR} ]
-	then
+	if [ ${BUILDDIR}/.compiled -nt "${WORKDIR}" ]; then
 		echo ">>> It appears that ${PN} is already compiled; skipping."
 		echo ">>> (clean to force compilation)"
 		trap SIGINT SIGQUIT
 		return
 	fi
-	if [ -d ${S} ]
-		then
-		cd ${S}
+	if [ -d "${S}" ]; then
+		cd "${S}"
 	fi
 	#our custom version of libtool uses $S and $D to fix
 	#invalid paths in .la files
@@ -645,38 +692,31 @@ dyn_compile() {
 	#some packages use an alternative to $S to build in, cause
 	#our libtool to create problematic .la files
 	export PWORKDIR="$WORKDIR"
-	#some users have $TMP/$TMPDIR to a custom dir in their home ...
-	#this will cause sandbox errors with some ./configure
-	#scripts, so set it to $T.
-	export TMP="${T}"
-	export TMPDIR="${T}"
 	src_compile 
 	#|| abort_compile "fail" 
 	cd ${BUILDDIR}
 	touch .compiled
-	if [ ! -e "build-info" ]
-	then
-		mkdir build-info
-	fi
 	cd build-info
-	echo "$CFLAGS"   > CFLAGS
-	echo "$CXXFLAGS" > CXXFLAGS
-	echo "$CC"       > CC
-	echo "$CXX"      > CXX
-	echo "$CHOST"    > CHOST
 	echo "$CBUILD"   > CBUILD
-	echo "$USE"      > USE
+	echo "$CC"       > CC
+	echo "$CDEPEND"  > CDEPEND
+	echo "$CFLAGS"   > CFLAGS
+	echo "$CHOST"    > CHOST
+	echo "$CXX"      > CXX
+	echo "$CXXFLAGS" > CXXFLAGS
+	echo "$DEPEND"   > DEPEND
+	echo "$IUSE"     > IUSE
 	echo "$LICENSE"  > LICENSE
 	echo "$CATEGORY" > CATEGORY
-	echo "$PF"       > PF
-	echo "$SLOT"     > SLOT
-	echo "$RDEPEND"  > RDEPEND
-	echo "$CDEPEND"  > CDEPEND
 	echo "$PDEPEND"  > PDEPEND
+	echo "$PF"       > PF
 	echo "$PROVIDE"  > PROVIDE
+	echo "$RDEPEND"  > RDEPEND
+	echo "$SLOT"     > SLOT
+	echo "$USE"      > USE
+	set | bzip2 -9 - > environment.bz2
 	cp ${EBUILD} ${PF}.ebuild
-	if [ -n "$DEBUGBUILD" ]
-	then
+	if has nostrip $FEATURES $RESTRICT; then
 		touch DEBUGBUILD
 	fi
 	trap SIGINT SIGQUIT
@@ -685,21 +725,19 @@ dyn_compile() {
 dyn_package() {
 	trap "abort_package" SIGINT SIGQUIT
 	cd ${BUILDDIR}/image
-	tar cvf ../bin.tar *
+	tar cpvf - * | bzip2 -f > ../bin.tar.bz2 || die "Failed to create tarball"
 	cd ..
-	bzip2 -f bin.tar
 	xpak build-info inf.xpak
 	tbz2tool join bin.tar.bz2 inf.xpak ${PF}.tbz2
-	mv ${PF}.tbz2 ${PKGDIR}/All
+	mv ${PF}.tbz2 ${PKGDIR}/All || die "Failed to move tbz2 to ${PKGDIR}/All"
 	rm -f inf.xpak bin.tar.bz2
-	if [ ! -d ${PKGDIR}/${CATEGORY} ]
-	then
+	if [ ! -d ${PKGDIR}/${CATEGORY} ]; then
 		install -d ${PKGDIR}/${CATEGORY}
 	fi
-	ln -sf ../All/${PF}.tbz2 ${PKGDIR}/${CATEGORY}/${PF}.tbz2
+	ln -sf ../All/${PF}.tbz2 ${PKGDIR}/${CATEGORY}/${PF}.tbz2 || die "Failed to create symlink in ${PKGDIR}/${CATEGORY}"
 	echo ">>> Done."
 	cd ${BUILDDIR}
-	touch .packaged
+	touch .packaged || die "Failed to 'touch .packaged' in ${BUILDDIR}"
 	trap SIGINT SIGQUIT
 }
 
@@ -707,9 +745,8 @@ dyn_install() {
 	trap "abort_install" SIGINT SIGQUIT
 	rm -rf ${BUILDDIR}/image
 	mkdir ${BUILDDIR}/image
-	if [ -d ${S} ]
-	then
-		cd ${S}
+	if [ -d "${S}" ]; then
+		cd "${S}"
 	fi
 	echo
 	echo ">>> Install ${PF} into ${D} category ${CATEGORY}"
@@ -719,11 +756,6 @@ dyn_install() {
 	#some packages uses an alternative to $S to build in, cause
 	#our libtool to create problematic .la files
 	export PWORKDIR="$WORKDIR"
-	#some users have $TMP/$TMPDIR to a custom dir in thier home ...
-	#this will cause sandbox errors with some ./configure
-	#scripts, so set it to $T.
-	export TMP="${T}"
-	export TMPDIR="${T}"
 	src_install 
 	#|| abort_install "fail"
 	prepall
@@ -742,7 +774,13 @@ dyn_install() {
 	if [[ $UNSAFE > 0 ]]; then
 		die "There are unsafe files. Portage will not install them."
 	fi
-
+	
+	find ${D}/ -user  portage -print0 | $XARGS -0 -n100 chown root
+	if [ "$USERLAND" == "BSD" ]; then
+		find ${D}/ -group portage -print0 | $XARGS -0 -n100 chgrp wheel
+	else	
+		find ${D}/ -group portage -print0 | $XARGS -0 -n100 chgrp root 
+	fi
 	echo ">>> Completed installing into ${D}"
 	echo
 	cd ${BUILDDIR}
@@ -750,7 +788,7 @@ dyn_install() {
 }
 
 dyn_spec() {
-	tar czf /usr/src/redhat/SOURCES/${PF}.tar.gz ${O}/${PF}.ebuild ${O}/files
+	tar czf /usr/src/redhat/SOURCES/${PF}.tar.gz ${O}/${PF}.ebuild ${O}/files || die "Failed to create base rpm tarball."
 
 	cat <<__END1__ > ${PF}.spec
 Summary: ${DESCRIPTION}
@@ -783,8 +821,8 @@ __END1__
 
 dyn_rpm() {
 	dyn_spec
-	rpm -bb ${PF}.spec
-	install -D /usr/src/redhat/RPMS/i386/${PN}-${PV}-${PR}.i386.rpm ${RPMDIR}/${CATEGORY}/${PN}-${PV}-${PR}.rpm
+	rpm -bb ${PF}.spec || die "Failed to integrate rpm spec file"
+	install -D /usr/src/redhat/RPMS/i386/${PN}-${PV}-${PR}.i386.rpm ${RPMDIR}/${CATEGORY}/${PN}-${PV}-${PR}.rpm || die "Failed to move rpm"
 }
 
 dyn_help() {
@@ -821,16 +859,14 @@ dyn_help() {
 	echo "  c++ flags   : ${CXXFLAGS}" 
 	echo "  make flags  : ${MAKEOPTS}" 
 	echo -n "  build mode  : "
-	if [ -n "${DEBUGBUILD}" ]
-	then
+	if has nostrip $FEATURES $RESTRICT;	then
 		echo "debug (large)"
 	else
 		echo "production (stripped)"
 	fi
 	echo "  merge to    : ${ROOT}" 
 	echo
-	if [ -n "$USE" ]
-	then
+	if [ -n "$USE" ]; then
 		echo "Additionally, support for the following optional features will be enabled:"
 		echo 
 		echo "  ${USE}"
@@ -862,7 +898,7 @@ debug-print() {
 		fi
 		
 		# default target
-		echo $1 >> ${T}/eclass-debug.log
+		echo "$1" >> ${T}/eclass-debug.log
 		# let the portage user own/write to this file
 		chmod g+w ${T}/eclass-debug.log &>/dev/null
 		
@@ -883,34 +919,88 @@ debug-print-section() {
 }
 
 # Sources all eclasses in parameters
+declare -ix ECLASS_DEPTH=0
 inherit() {
-	unset INHERITED
+	ECLASS_DEPTH=$(($ECLASS_DEPTH + 1))
+	if [[ $ECLASS_DEPTH > 1 ]]; then
+		debug-print "*** Multiple Inheritence (Level: ${ECLASS_DEPTH})"
+	fi
+
 	local location
-	while [ "$1" ]
-	do
+
+	while [ "$1" ]; do
 		location="${ECLASSDIR}/${1}.eclass"
+
+		# PECLASS is used to restore the ECLASS var after recursion.
 		PECLASS="$ECLASS"
 		export ECLASS="$1"
 
 		# any future resolution code goes here
-		if [ -n "$PORTDIR_OVERLAY" ]
-		then
-			olocation="${PORTDIR_OVERLAY}/eclass/${1}.eclass"
-			if [ -e "$olocation" ]; then
-				location="${olocation}"
-			fi
+		if [ -n "$PORTDIR_OVERLAY" ]; then
+			local overlay
+			for overlay in ${PORTDIR_OVERLAY}; do
+				olocation="${overlay}/eclass/${1}.eclass"
+				if [ -e "$olocation" ]; then
+					location="${olocation}"
+					debug-print "  eclass exists: ${location}"
+					break
+				fi
+			done
 		fi
 		debug-print "inherit: $1 -> $location"
-		source "$location" || die "died sourcing $location in inherit()"
-		ECLASS="$PECLASS"
-		unset PECLASS
 
-		if ! has $1 $INHERITED &>/dev/null; then
-			export INHERITED="$INHERITED $1"
-		fi
+		#We need to back up the value of DEPEND and RDEPEND to B_DEPEND and B_RDEPEND
+		#(if set).. and then restore them after the inherit call.
+	
+		#turn off glob expansion
+		set -f
+
+		# Retain the old data and restore it later.
+		unset B_DEPEND B_RDEPEND B_CDEPEND B_PDEPEND
+		[ "${DEPEND-unset}"  != "unset" ] && B_DEPEND="${DEPEND}"
+		[ "${RDEPEND-unset}" != "unset" ] && B_RDEPEND="${RDEPEND}"
+		[ "${CDEPEND-unset}" != "unset" ] && B_CDEPEND="${CDEPEND}"
+		[ "${PDEPEND-unset}" != "unset" ] && B_PDEPEND="${PDEPEND}"
+		unset   DEPEND   RDEPEND   CDEPEND   PDEPEND
+		#turn on glob expansion
+		set +f
+		
+		source "$location" || export ERRORMSG="died sourcing $location in inherit()"
+		[ -z "${ERRORMSG}" ] || die "${ERRORMSG}"
+		
+		#turn off glob expansion
+		set -f
+
+		# If each var has a value, append it to the global variable E_* to
+		# be applied after everything is finished. New incremental behavior.
+		[ "${DEPEND-unset}"  != "unset" ] && export E_DEPEND="${E_DEPEND} ${DEPEND}"
+		[ "${RDEPEND-unset}" != "unset" ] && export E_RDEPEND="${E_RDEPEND} ${RDEPEND}"
+		[ "${CDEPEND-unset}" != "unset" ] && export E_CDEPEND="${E_CDEPEND} ${CDEPEND}"
+		[ "${PDEPEND-unset}" != "unset" ] && export E_PDEPEND="${E_PDEPEND} ${PDEPEND}"
+
+		[ "${B_DEPEND-unset}"  != "unset" ] && DEPEND="${B_DEPEND}"
+		[ "${B_DEPEND-unset}"  != "unset" ] || unset DEPEND
+
+		[ "${B_RDEPEND-unset}" != "unset" ] && RDEPEND="${B_RDEPEND}"
+		[ "${B_RDEPEND-unset}" != "unset" ] || unset RDEPEND
+
+		[ "${B_CDEPEND-unset}" != "unset" ] && CDEPEND="${B_CDEPEND}"
+		[ "${B_CDEPEND-unset}" != "unset" ] || unset CDEPEND
+
+		[ "${B_PDEPEND-unset}" != "unset" ] && PDEPEND="${B_PDEPEND}"
+		[ "${B_PDEPEND-unset}" != "unset" ] || unset PDEPEND
+
+		#turn on glob expansion
+ 		set +f
+		
+		has $1 $INHERITED || export INHERITED="$INHERITED $1"
+
+		export ECLASS="$PECLASS"
+		unset PECLASS
 
 		shift
 	done
+	ECLASS_DEPTH=$(($ECLASS_DEPTH - 1))
 }
 
 # Exports stub functions that call the eclass's functions, thereby making them default.
@@ -929,32 +1019,73 @@ EXPORT_FUNCTIONS() {
 	done
 }
 
-# adds all parameters to DEPEND and RDEPEND
+# adds all parameters to E_DEPEND and E_RDEPEND, which get added to DEPEND
+# and RDEPEND after the ebuild has been processed. This is important to
+# allow users to use DEPEND="foo" without frying dependencies added by an
+# earlier inherit. It also allows RDEPEND to work properly, since a lot
+# of ebuilds assume that an unset RDEPEND gets its value from DEPEND.
+# Without eclasses, this is true. But with them, the eclass may set
+# RDEPEND itself (or at least used to) which would prevent RDEPEND from
+# getting its value from DEPEND. This is a side-effect that made eclasses
+# have unreliable dependencies.
+
 newdepend() {
 	debug-print-function newdepend $*
-	debug-print "newdepend: DEPEND=$DEPEND RDEPEND=$RDEPEND"
+	debug-print "newdepend: E_DEPEND=$E_DEPEND E_RDEPEND=$E_RDEPEND"
 
 	while [ -n "$1" ]; do
 		case $1 in
 		"/autotools")
-			DEPEND="${DEPEND} sys-devel/autoconf sys-devel/automake sys-devel/make"
+			do_newdepend DEPEND sys-devel/autoconf sys-devel/automake sys-devel/make
 			;;
 		"/c")
-			DEPEND="${DEPEND} sys-devel/gcc virtual/glibc"
-			RDEPEND="${RDEPEND} virtual/glibc"
+			do_newdepend DEPEND sys-devel/gcc virtual/glibc
+			do_newdepend RDEPEND virtual/glibc
 			;;
 		*)
-			DEPEND="$DEPEND $1"
-			if [ -z "$RDEPEND" ] && [ "${RDEPEND-unset}" == "unset" ]; then
-				export RDEPEND="$DEPEND"
-			fi
-			RDEPEND="$RDEPEND $1"
+			do_newdepend DEPEND $1
 			;;
 		esac
 		shift
 	done
 }
 
+newrdepend() {
+	debug-print-function newrdepend $*
+	do_newdepend RDEPEND $1
+}
+
+newcdepend() {
+	debug-print-function newcdepend $*
+	do_newdepend CDEPEND $1
+}
+
+newpdepend() {
+	debug-print-function newpdepend $*
+	do_newdepend PDEPEND $1
+}
+
+do_newdepend() {
+	# This function does a generic change determining whether we're in an
+	# eclass or not. If we are, we change the E_* variables for deps.
+	debug-print-function do_newdepend $*
+	[ -z "$1" ] && die "do_newdepend without arguments"
+
+	# Grab what we're affecting... Figure out if we're affecting eclasses.
+	[[ ${ECLASS_DEPTH} > 0 ]] && TARGET="E_$1"
+	[[ ${ECLASS_DEPTH} > 0 ]] || TARGET="$1"
+	shift # $1 was a variable name.
+
+	while [ -n "$1" ]; do
+		# This bit of evil takes TARGET and uses it to evaluate down to a
+		# variable. This is a sneaky way to make this infinately expandable.
+		# The normal translation of this would look something like this:
+		# E_DEPEND="${E_DEPEND} $1"  ::::::  Cool, huh? :)
+		eval export ${TARGET}=\"\${${TARGET}} $1\"
+		shift
+	done
+}
+
 # === === === === === === === === === === === === === === === === === ===
 # === === === === === functions end, main part begins === === === === ===
 # === === === === === functions end, main part begins === === === === ===
@@ -965,33 +1096,6 @@ if [ "$*" != "depend" ] && [ "$*" != "clean" ]; then
 	cd ${PORTAGE_TMPDIR} &> /dev/null
 	cd ${BUILD_PREFIX} &> /dev/null
 
-	if [ -f ${T}/successful ]; then
-		rm -f ${T}/successful
-	fi
-
-	# Hurray for per-ebuild logging.
-	if [ ! -z "${PORT_LOGDIR}" ]; then
-		if [ -z "${PORT_LOGGING}" ]; then
-			export PORT_LOGGING=1
-			export SANDBOX_WRITE="$SANDBOX_WRITE:${PORT_LOGDIR}"
-			install -d ${PORT_LOGDIR} &>/dev/null
-			chown root:portage ${PORT_LOGDIR} &>/dev/null
-			chmod g+rwxs ${PORT_LOGDIR} &> /dev/null
-			touch "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log" &> /dev/null
-			chmod g+w "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log" &> /dev/null
-			$0 $* 2>&1 | tee -a "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log"
-			if [ "$?" != "0" ]; then
-				exit 1
-			fi
-			if [ -f ${T}/successful ]; then
-				rm -f ${T}/successful
-				exit 0
-			else
-				exit 1
-			fi
-		fi
-	fi
-
 	if [ `id -nu` == "portage" ] ; then
 		export USER=portage
 	fi
@@ -1004,9 +1108,7 @@ if [ "$*" != "depend" ] && [ "$*" != "clean" ]; then
 				PATH="$(echo ${PATH} | sed 's/:[^:]*distcc[^:]*:/:/;s/^[^:]*distcc[^:]*://;s/:[^:]*distcc[^:]*$//')"
 			fi
 			export PATH="/usr/lib/distcc/bin:${PATH}"
-			[ -z "${DISTCC_HOSTS}" ] && DISTCC_HOSTS="localhost"
 			[ ! -z "${DISTCC_LOG}" ] && addwrite "$(dirname ${DISTCC_LOG})"
-			export DISTCC_HOSTS
 		elif which distcc &>/dev/null; then
 			export CC="distcc $CC"
 			export CXX="distcc $CXX"
@@ -1038,15 +1140,23 @@ fi # "$*"!="depend" && "$*"!="clean"
 
 export SANDBOX_ON="1"
 export S=${WORKDIR}/${P}
+
+unset   DEPEND   RDEPEND   CDEPEND   PDEPEND
+unset E_DEPEND E_RDEPEND E_CDEPEND E_PDEPEND
+
 source ${EBUILD} || die "error sourcing ebuild"
+[ -z "${ERRORMSG}" ] || die "${ERRORMSG}"
+
 #a reasonable default for $S
 if [ "$S" = "" ]; then
 	export S=${WORKDIR}/${P}
 fi
-if [ "${RESTRICT/nostrip/}" != "${RESTRICT}" ]
-then
-	export DEBUGBUILD="yes"
-fi
+
+#some users have $TMP/$TMPDIR to a custom dir in their home ...
+#this will cause sandbox errors with some ./configure
+#scripts, so set it to $T.
+export TMP="${T}"
+export TMPDIR="${T}"
 
 # Note: this next line is not the same as export RDEPEND=${RDEPEND:-${DEPEND}}
 # That will test for unset *or* NULL ("").  We want just to set for unset...
@@ -1059,18 +1169,23 @@ if [ "${RDEPEND-unset}" == "unset" ]; then
 	export RDEPEND=${DEPEND}
 	debug-print "RDEPEND: not set... Setting to: ${DEPEND}"
 fi
+
+#add in dependency info from eclasses
+DEPEND="$DEPEND $E_DEPEND"
+RDEPEND="$RDEPEND $E_RDEPEND"
+CDEPEND="$CDEPEND $E_CDEPEND"
+PDEPEND="$PDEPEND $E_PDEPEND"
+
 set +f
 
-for myarg in $*
-do
+for myarg in $*; do
 	case $myarg in
 	nofetch)
 		pkg_nofetch
 		;;
 	prerm|postrm|preinst|postinst|config)
 		export SANDBOX_ON="0"
-		if [ "$PORTAGE_DEBUG" != "1" ]
-		then
+		if [ "$PORTAGE_DEBUG" != "1" ]; then
 			pkg_${myarg}
 			#Allow non-zero return codes since they can be caused by &&
 		else
@@ -1081,14 +1196,12 @@ do
 		fi
 		;;
 	unpack|compile|clean|install)
-		if [ ${SANDBOX_DISABLED="0"} = "0" ]
-		then
+		if [ "${SANDBOX_DISABLED="0"}" == "0" ]; then
 			export SANDBOX_ON="1"
 		else
 			export SANDBOX_ON="0"
 		fi
-		if [ "$PORTAGE_DEBUG" != "1" ]
-		then
+		if [ "$PORTAGE_DEBUG" != "1" ]; then
 			dyn_${myarg}
 			#Allow non-zero return codes since they can be caused by &&
 		else
@@ -1114,8 +1227,7 @@ do
 		;;
 	package|rpm)
 		export SANDBOX_ON="0"
-		if [ "$PORTAGE_DEBUG" != "1" ]
-		then
+		if [ "$PORTAGE_DEBUG" != "1" ]; then
 			dyn_${myarg}
 		else
 			set -x
@@ -1128,25 +1240,24 @@ do
 		set -f
 		#the extra `echo` commands remove newlines
 		dbkey=${PORTAGE_CACHEDIR}/${CATEGORY}/${PF}
-		if [ ! -d ${PORTAGE_CACHEDIR}/${CATEGORY} ]
-		then
+		if [ ! -d ${PORTAGE_CACHEDIR}/${CATEGORY} ]; then
 			install -d -g ${PORTAGE_GID} -m4775 ${PORTAGE_CACHEDIR}/${CATEGORY}
 		fi
 		# Make it group writable. 666&~002==664
 		umask 002
-		echo `echo "$DEPEND"` > $dbkey
-		echo `echo "$RDEPEND"` >> $dbkey
-		echo `echo "$SLOT"` >> $dbkey
-		echo `echo "$SRC_URI"` >> $dbkey
-		echo `echo "$RESTRICT"` >> $dbkey
-		echo `echo "$HOMEPAGE"` >> $dbkey
-		echo `echo "$LICENSE"` >> $dbkey
+		echo `echo "$DEPEND"`       > $dbkey
+		echo `echo "$RDEPEND"`     >> $dbkey
+		echo `echo "$SLOT"`        >> $dbkey
+		echo `echo "$SRC_URI"`     >> $dbkey
+		echo `echo "$RESTRICT"`    >> $dbkey
+		echo `echo "$HOMEPAGE"`    >> $dbkey
+		echo `echo "$LICENSE"`     >> $dbkey
 		echo `echo "$DESCRIPTION"` >> $dbkey
-		echo `echo "$KEYWORDS"` >> $dbkey
-		echo `echo "$INHERITED"` >> $dbkey
-		echo `echo "$IUSE"` >> $dbkey
-		echo `echo "$CDEPEND"` >> $dbkey
-		echo `echo "$PDEPEND"` >> $dbkey
+		echo `echo "$KEYWORDS"`    >> $dbkey
+		echo `echo "$INHERITED"`   >> $dbkey
+		echo `echo "$IUSE"`        >> $dbkey
+		echo `echo "$CDEPEND"`     >> $dbkey
+		echo `echo "$PDEPEND"`     >> $dbkey
 		set +f
 		#make sure it is writable by our group:
 		exit 0
@@ -1159,8 +1270,7 @@ do
 		exit 1
 		;;
 	esac
-	if [ $? -ne 0 ]
-	then
+	if [ $? -ne 0 ]; then
 		exit 1
 	fi
 done
@@ -1168,7 +1278,7 @@ done
 if [ "$myarg" != "clean" ]; then
 	# Save current environment and touch a success file. (echo for success)
 	umask 002
-	set | egrep -v "^SANDBOX" > ${T}/environment 2>/dev/null
+	set | egrep -v "^SANDBOX_" > ${T}/environment 2>/dev/null
 	chown portage:portage ${T}/environment &>/dev/null
 	chmod g+w ${T}/environment &>/dev/null
 fi
