@@ -1,7 +1,7 @@
 #!/usr/bin/env python2.2
 # Copyright 1999-2003 Gentoo Technologies, Inc.
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /home/cvsroot/gentoo-src/portage/bin/emergehelp.py,v 1.11 2003/02/27 11:31:00 carpaski Exp $
+# $Header: /home/cvsroot/gentoo-src/portage/bin/emergehelp.py,v 1.14 2003/05/20 09:15:59 carpaski Exp $
 
 import os,sys
 from output import *
@@ -13,9 +13,10 @@ def shorthelp():
 	print "   "+turquoise("emerge")+" [ "+green("options")+" ] [ "+green("action")+" ] [ "+turquoise("ebuildfile")+" | "+turquoise("tbz2file")+" | "+turquoise("dependency")+" ] [ ... ]"
 	print "   "+turquoise("emerge")+" [ "+green("options")+" ] [ "+green("action")+" ] < "+turquoise("system")+" | "+turquoise("world")+" >"
 	print "   "+turquoise("emerge")+" < "+turquoise("sync")+" | "+turquoise("info")+" >"
-	print "   "+turquoise("emerge")+" "+turquoise("--resume")+" ["+green("--pretend")+"]"
+	print "   "+turquoise("emerge")+" "+turquoise("--resume")+" ["+green("--pretend")+" | "+green("--skipfirst")+"]"
 	print "   "+turquoise("emerge")+" "+turquoise("help")+" [ "+green("system")+" | "+green("config")+" | "+green("sync")+" ] "
 	print bold("Options:")+" "+green("-")+"["+green("bcCdDefhikKlnoOpPsSuUvV")+"] ["+green("--oneshot")+"] ["+green("--noconfmem")+"]"
+	print      "                                    ["+green("--columns")+"] ["+green("--nospinner")+"]"
 	print bold("Actions:")+" [ "+green("clean")+" | "+green("depclean")+" | "+green("inject")+" | "+green("prune")+" | "+green("regen")+" | "+green("search")+" | "+green("unmerge")+" ]"
 	print
 
@@ -129,7 +130,7 @@ def help(myaction,myopts,havecolor=1):
 		print
 		print "       "+green("--buildpkgonly")+" ("+green("-B")+" short option)"
 		print "              Creates binary a binary package, but does not merge it to the"
-		print "              system. This has the restriction that unsatisfied dependancies"
+		print "              system. This has the restriction that unsatisfied dependencies"
 		print "              must not exist for the desired package as they cannot be used if"
 		print "              they do not exist on the system."
 		print
@@ -137,11 +138,16 @@ def help(myaction,myopts,havecolor=1):
 		print "              When pretending, also display the ChangeLog entries for packages"
 		print "              that will be upgraded."
 		print
+		print "       "+green("--columns")
+		print "              Display the pretend output in a tabular form. Versions are"
+		print "              aligned vertically."
+		print
 		print "       "+green("--debug")+" ("+green("-d")+" short option)"
 		print "              Tell emerge to run the ebuild command in --debug mode. In this"
 		print "              mode, the bash build environment will run with the -x option,"
 		print "              causing it to output verbose debug information print to stdout."
-		print "              --debug is great for finding bash syntax errors."
+		print "              --debug is great for finding bash syntax errors as providing"
+		print "              very verbose information about the dependency and build process."
 		print
 		print "       "+green("--deep")+" ("+green("-D")+" short option)"
 		print "              When used in conjunction with --update, this flag forces emerge"
@@ -177,9 +183,12 @@ def help(myaction,myopts,havecolor=1):
 		print "       "+green("--noreplace")+" ("+green("-n")+" short option)"
 		print "              Skip the packages specified on the command-line that have"
 		print "              already been installed.  Without this option, any packages,"
-		print "              ebuilds, or deps you specify on on the command-line *will* cause"
+		print "              ebuilds, or deps you specify on the command-line *will* cause"
 		print "              Portage to remerge the package, even if it is already installed."
 		print "              Note that Portage won't remerge dependencies by default."
+		print 
+		print "       "+green("--nospinner")
+		print "              Disables the spinner irregardless of terminal type."
 		print
 		print "       "+green("--oneshot")
 		print "              Emerge as normal, but don't add packages to the world profile for"
@@ -200,6 +209,10 @@ def help(myaction,myopts,havecolor=1):
 		print "              use flags to be printed out accompanied by a '+' for enabled"
 		print "              and a '-' for disabled flags."
 		print
+		print "       "+green("--quiet")+" ("+green("-q")+" short option)"
+		print "              Effects vary, but the general outcome is a reduced or condensed"
+		print "              output from portage's displays."
+		print
 		print "       "+green("--resume")
 		print "              Resumes the last merge operation. Can be treated just like a"
 		print "              regular merge as --pretend and other options work along side."
@@ -214,6 +227,13 @@ def help(myaction,myopts,havecolor=1):
 		print "                emerge -S applet"
 		print "                emerge -S 'perl.*module'"
 		print
+		print "       "+green("--skipfirst")
+		print "              This option is only valid in a resume situation. It removes the"
+		print "              first package in the resume list so that a merge may continue in"
+		print "              the presence of an uncorrectable or inconsequential error. This"
+		print "              should only be used in cases where skipping the package will not"
+		print "              result in failed dependencies."
+		print
 		print "       "+green("--update")+" ("+green("-u")+" short option)"
 		print "              Updates packages to the best version available, which may not"
 		print "              always be the highest version number due to masking for testing"
@@ -238,7 +258,14 @@ def help(myaction,myopts,havecolor=1):
 		print "              available at the time of dependency calculation."
 		print
 		print "       "+green("--verbose")+" ("+green("-v")+" short option)"
-		print "              Tell emerge to run in verbose mode."
+		print "              Effects vary, but the general outcome is an increased or expanded"
+		print "              display of content in portage's displays."
+		print
+		print "       "+green("--version")+" ("+green("-V")+" short option)"
+		print "              Displays the currently installed version of portage along with"
+		print "              other information useful for quick reference on a system. See"
+		print "              "+bold("emerge info")+" for more advanced information."
+		print
 	elif myaction in ["rsync","sync"]:
 		print
 		print bold("Usage: ")+turquoise("emerge")+" "+turquoise("sync")
